'''This file provides some 'castle' building functionalities. The name castle,
here, is loosely based on an LPMUD castle.'''

# Pygame libs #
import pygame

# Game libs #
import data

# Constants #
MAP_NAME = "map.txt"

# Exits #
N_EXIT = pygame.rect.Rect(280, 320, 80, 10)
S_EXIT = pygame.rect.Rect(280, 470, 80, 10)
E_EXIT = pygame.rect.Rect(540, 360, 50, 50)
W_EXIT = pygame.rect.Rect(50,  360, 50, 50)
N_ENTRANCE = (320, 330)
S_ENTRANCE = (320, 460)
E_ENTRANCE = (530, 385)
W_ENTRANCE = (110, 385)

class Exit(object):
    def __init__(self, bounding_box, entrance, next_room):
        self.bounding_box = bounding_box
        self.entrance     = entrance
        self.next_room    = next_room
    
    def has_point(self, (x, y)):
        return self.bounding_box.collidepoint((x, y))
    
    def get_entrance(self):
        return self.entrance
    
    def get_next_room(self):
        return self.next_room

class NorthExit(Exit):
    def __init__(self, next_room):
        Exit.__init__(self, N_EXIT, S_ENTRANCE, next_room)

class SouthExit(Exit):
    def __init__(self, next_room):
        Exit.__init__(self, S_EXIT, N_ENTRANCE, next_room)

class EastExit(Exit):
    def __init__(self, next_room):
        Exit.__init__(self, E_EXIT, W_ENTRANCE, next_room)

class WestExit(Exit):
    def __init__(self, next_room):
        Exit.__init__(self, W_EXIT, E_ENTRANCE, next_room)

# Rooms #
class Room(object):
    def __init__(self, (x, y), (n, s, e, w)):
        (self.x, self.y) = x, y
        (self.n, self.s, self.e, self.w) = (n, s, e, w)
        self.exits = []
        if n:
            self.exits.append(NorthExit((x, y-1)))
        if s:
            self.exits.append(SouthExit((x, y+1)))
        if e:
            self.exits.append(EastExit((x+1, y)))
        if w:
            self.exits.append(WestExit((x-1, y)))
        self.image = self.get_background_image()
    
    def draw(self, target_surface, (x, y)=(0, 0)):
        target_surface.blit(self.image, (x, y))
    
    def get_background_file(self):
        return "room_{0}{1}.png".format(self.x, self.y)
    
    def get_background_image(self):
        return data.load_image(self.get_background_file())
    
    def get_coords(self):
        return (self.x, self.y)
    
    def get_exits(self):
        return self.exits
    
    def has_exit(self, name):
        return self.__dict__[name]

# The castle #
class Castle(object):
    def __init__(self):
        self.rooms = []
        self.rooms.append([])
        self.rooms[0].append(Room((0, 0), (False, True,  True, False)))
        self.rooms[0].append(Room((0, 1), (True,  False, True, False)))
        self.rooms.append([])
        self.rooms[1].append(Room((1, 0), (False, True,  False, True)))
        self.rooms[1].append(Room((1, 1), (True,  False, False, True)))
    
    def load_map(self, file_name):
        pass

    def get_room(self, (x, y)):
        return self.rooms[x][y]