'''Data handling functionalities'''

# Default libs #
import os
from functools import wraps

# Pygame libs #
import pygame

# Data #
resources = {}

# Decorators #
def caching(function):
    '''This decorator performs caching of loaded resources.'''
    @wraps(function)
    def cache_function(resource_name):
        if not resources.has_key(resource_name):
            resources[resource_name] = function(resource_name)
            print "Loaded resource %s - %s" %(resource_name, resources[resource_name])
        return resources[resource_name]
    return cache_function

# Functions #

@caching
def load_image_transparency(resource_name):
    '''Returns an image with per-pixel alpha values. Uses caching.'''
    surface  = pygame.image.load(os.path.join("data", "image", resource_name))
    resource = surface.convert()
    resource.set_colorkey((255, 0, 255))
    return resource

@caching
def load_image(resource_name):
    '''Returns an image with 'color-keyed' alpha pixels. Uses caching.'''
    surface  = pygame.image.load(os.path.join("data", "image", resource_name))
    resource = surface.convert()
    return resource

@caching
def load_sound(resource_name):
    '''Returns a sound. Uses caching.'''
    return pygame.mixer.Sound(os.path.join("data", "sound", resource_name))

def load_music(resource_name):
    '''Loads a music file.'''
    return pygame.mixer.music.load(os.path.join("data", "music", resource_name))